/**
 * @(#)pms12.cpp 2000/01/01
 *
 * @author Tognon Stefano
 * @version 1.00
 *
 * This program automatically generates random prime magic squares of order 12.
 * The square is printed in standard output, while the other information are
 * printed in standard error.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#include <iostream.h>
#include <stdlib.h>
#include <sys/timeb.h>

typedef unsigned int word;

const int ORDER=15;     // square order
const int COMB=32768;   // 2 at pow of order
const word MAGIC=9635;  // magic constant
const word NMAGIC=1695; // normal magic constant

// number of max spaces between output values
const int MAX_BLANK=5;

// list of sequential primes numbers
const word PRIME[ORDER*ORDER]={
          1,    3,    5,    7,   11,   13,   17,   19,   23,   29,   31,
         37,   41,   43,   47,   53,   59,   61,   67,   71,   73,   79,
         83,   89,   97,  101,  103,  107,  109,  113,  127,  131,  137,
        139,  149,  151,  157,  163,  167,  173,  179,  181,  191,  193,
        197,  199,  211,  223,  227,  229,  233,  239,  241,  251,  257,
        263,  269,  271,  277,  281,  283,  293,  307,  311,  313,  317,
        331,  337,  347,  349,  353,  359,  367,  373,  379,  383,  389,
        397,  401,  409,  419,  421,  431,  433,  439,  443,  449,  457,
        461,  463,  467,  479,  487,  491,  499,  503,  509,  521,  523,
        541,  547,  557,  563,  569,  571,  577,  587,  593,  599,  601,
        607,  613,  617,  619,  631,  641,  643,  647,  653,  659,  661,
        673,  677,  683,  691,  701,  709,  719,  727,  733,  739,  743,
        751,  757,  761,  769,  773,  787,  797,  809,  811,  821,  823,
        827,  829,  839,  853,  857,  859,  863,  877,  881,  883,  887,
        907,  911,  919,  929,  937,  941,  947,  953,  967,  971,  977,
        983,  991,  997, 1009, 1013, 1019, 1021, 1031, 1033, 1039, 1049,
       1051, 1061, 1063, 1069, 1087, 1091, 1093, 1097, 1103, 1109, 1117,
       1123, 1129, 1151, 1153, 1163, 1171, 1181, 1187, 1193, 1201, 1213,
       1217, 1223, 1229, 1231, 1237, 1249, 1259, 1277, 1279, 1283, 1289,
       1291, 1297, 1301, 1303, 1307, 1319, 1321, 1327, 1361, 1367, 1373,
       1381, 1399, 1409, 1423, 1427};

// this is a union of bits and the corresponding int value.
// if the program never produce results, probably your compiler manage this
// union in a different way. If we increase 'counter' then some bits may
// changes their value.
union unionBits {
  int counter;
  struct {
    unsigned bit0 :1;
    unsigned bit1 :1;
    unsigned bit2 :1;
    unsigned bit3 :1;
    unsigned bit4 :1;
    unsigned bit5 :1;
    unsigned bit6 :1;
    unsigned bit7 :1;
    unsigned bit8 :1;
    unsigned bit9 :1;
    unsigned bit10 :1;
    unsigned bit11 :1;
    unsigned bit12 :1;
    unsigned bit13 :1;
    unsigned bit14 :1;
  } bit;
};

// the square representation
struct square {
  word cell[ORDER][ORDER];    // cells elements
  word row[ORDER];            // rows sums
  word col[ORDER];            // columns sums
  word diag1;                 // diag 1 \ sums
  word diag2;                 // diag 2 / sums
};

// square of normal values (1,2,3...)
square normalSquare;

// square of primes values (1,3,5,..)
square primeSquare;

// actual time out for timing loop
long int timeOut;

// time out limit of timing loop
long int timeOutLimit=0;

// note: these defines works ok with even order

#define UP_SEQ {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14}
#define DN_SEQ {14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0}

#define IJ(a) (i==a || j==(ORDER-a-1) || j==a || i==(ORDER-a-1))

#define POS(a) (pos[a]!=a)
#define POSES   POS(0) && POS(1) && POS(2) && POS(3) && \
                POS(4) && POS(5) && POS(6) && (pos[7]==7) && \
                POS(8) && POS(9) && POS(10) && POS(11) && \
                POS(12) && POS(13) && POS(14)

#define VALUE(a, sq) (sq.cell[pos[a]][a])
#define VALUES(b) (VALUE(0, b)+ VALUE(1, b)+ VALUE(2, b)+ VALUE(3, b)+ \
                   VALUE(4, b)+ VALUE(5, b)+ VALUE(6, b)+ VALUE(7, b)+ \
                   VALUE(8, b)+ VALUE(9, b)+ VALUE(10, b)+ VALUE(11, b) +\
                   VALUE(12, b)+ VALUE(13, b)+ VALUE(14, b))

#define IF_SWAP if (bits.bit.bit0) swap(sq, sn, i, 0, j, 0); \
                if (bits.bit.bit1) swap(sq, sn, i, 1, j, 1); \
                if (bits.bit.bit2) swap(sq, sn, i, 2, j, 2); \
                if (bits.bit.bit3) swap(sq, sn, i, 3, j, 3); \
                if (bits.bit.bit4) swap(sq, sn, i, 4, j, 4); \
                if (bits.bit.bit5) swap(sq, sn, i, 5, j, 5); \
                if (bits.bit.bit6) swap(sq, sn, i, 6, j, 6); \
                if (bits.bit.bit7) swap(sq, sn, i, 7, j, 7); \
                if (bits.bit.bit8) swap(sq, sn, i, 8, j, 8); \
                if (bits.bit.bit9) swap(sq, sn, i, 9, j, 9); \
                if (bits.bit.bit10) swap(sq, sn, i, 10, j, 10); \
                if (bits.bit.bit11) swap(sq, sn, i, 11, j, 11); \
                if (bits.bit.bit12) swap(sq, sn, i, 12, j, 12); \
                if (bits.bit.bit13) swap(sq, sn, i, 13, j, 13); \
                if (bits.bit.bit14) swap(sq, sn, i, 14, j, 14);

#define SUM_C(sq) sq.cell[index[bits.bit.bit0]][0]+ \
                  sq.cell[index[bits.bit.bit1]][1]+ \
                  sq.cell[index[bits.bit.bit2]][2]+ \
                  sq.cell[index[bits.bit.bit3]][3]+ \
                  sq.cell[index[bits.bit.bit4]][4]+ \
                  sq.cell[index[bits.bit.bit5]][5]+ \
                  sq.cell[index[bits.bit.bit6]][6]+ \
                  sq.cell[index[bits.bit.bit7]][7]+ \
                  sq.cell[index[bits.bit.bit8]][8]+ \
                  sq.cell[index[bits.bit.bit9]][9]+ \
                  sq.cell[index[bits.bit.bit10]][10]+ \
                  sq.cell[index[bits.bit.bit11]][11]+ \
                  sq.cell[index[bits.bit.bit12]][12]+ \
                  sq.cell[index[bits.bit.bit13]][13]+ \
                  sq.cell[index[bits.bit.bit14]][14]

#define SUM_CELL  SUM_C(sq)
#define SUM_NCELL SUM_C(sn)

#define BIT_IJ (bits.bit.bit0 && IJ(0)) || \
               (bits.bit.bit1 && IJ(1)) || \
               (bits.bit.bit2 && IJ(2)) || \
               (bits.bit.bit3 && IJ(3)) || \
               (bits.bit.bit4 && IJ(4)) || \
               (bits.bit.bit5 && IJ(5)) || \
               (bits.bit.bit6 && IJ(6)) || \
               (bits.bit.bit7 && IJ(7)) || \
               (bits.bit.bit8 && IJ(8)) || \
               (bits.bit.bit9 && IJ(9)) || \
               (bits.bit.bit10 && IJ(10)) || \
               (bits.bit.bit11 && IJ(11)) || \
               (bits.bit.bit12 && IJ(12)) || \
               (bits.bit.bit13 && IJ(13)) || \
               (bits.bit.bit14 && IJ(14))

/**
 * Erase the square.
 *
 * @param sq the square to fill with 0
 */
void eraseSquare(square &sq) {
  for (int i=0; i<ORDER; i++) {
    for (int j=0; j<ORDER; j++)
      sq.cell[i][j]=0u;                            // fill the cells element
    sq.row[i]=0u;                                  // fill the rows sum
    sq.col[i]=0u;                                  // fill the columns sum
  }
  sq.diag1=0u;                                     // fill diagonal \
  sq.diag2=0u;                                     // fill diagonal /
}

/**
 * Init the square with random normal numbers
 *
 * @param normal the square to
 */
void initRandom(square &sq) {
  int i;  // row index
  int j;  // column index

  for (int n=1; n<=ORDER*ORDER; n++) {
    do {
      i=(int)(drand48()*ORDER);                    // generate an row index
      j=(int)(drand48()*ORDER);                    // generate an column index
    } while (sq.cell[i][j]!=0);                    // is empty?
    sq.cell[i][j]=n;                               // mark with actual value
  }
}

/**
 * Write the square content to standard output (use > to put into a file)
 *
 * @param sq the square to write
 */
void writeSquare(square &sq) {
  word tmp;

  for (int i=0; i<ORDER; i++)  {
    for (int j=0; j<ORDER; j++) {
      tmp=sq.cell[i][j];
      cout << tmp;
      for (int s=1; s<MAX_BLANK; s++)              // make an formatted output
        if ((tmp/=10)==0)
          cout << " ";
    }
    cout << endl;
  }
  cout << endl;
}

/**
 * Transform a normal square to the corresponding prime
 *
 * @param sn the input normal square
 * @param sp the output prime square
 */
void transformToPrime(square &sn, square &sp) {
  for (int i=0; i<ORDER; i++)
    for (int j=0; j<ORDER; j++)
      sp.cell[i][j]=PRIME[sn.cell[i][j]-1];
}

/**
 * Calculate the sum of rows, columns and diagonals of the passed square
 *
 * @param sq the square to sum
 */
void sumSquare(square &sq) {
  sq.diag1=0;                                         // erase previous value
  sq.diag2=0;                                         // erase previous value

  for (int i=0; i<ORDER; i++) {
    sq.row[i]=0;                                      // erase previous value
    sq.col[i]=0;                                      // erase previous value
    sq.diag1+=sq.cell[i][i];                          // sum \ diagonal
    sq.diag2+=sq.cell[i][ORDER-i-1];                  // sum / diagonal
  }

  for (int i=0; i<ORDER; i++)                         // row index
    for (int j=0; j<ORDER; j++) {                     // column index
      sq.row[i]+=sq.cell[i][j];                       // sum row
      sq.col[j]+=sq.cell[i][j];                       // sum column
    }
}

/**
 * Calculate actual millisecond time
 *
 * @return the millisecond of actual time
 */
long double actualTime() {
  // actual time
  timeb time;

  ftime(&time);
  return (long double)(time.time*1000+time.millitm);
}

/**
 * Verify that a square is magic.
 *
 * @param sq the square to verify
 * @param sn the normal square
 * @return true if verify ok
 */
bool verifyMagic(square &sq, square &sn) {
  int i;
  bool res=true;

  sumSquare(sq);                                   // calculate the sums
  sumSquare(sn);

  cerr << "Verify that square is magic:" << endl
       << "rows\t";
  for (int i=0; i<ORDER; i++) {
    if (sq.row[i]==MAGIC) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr  << endl << "columns\t";
  for (int i=0; i<ORDER; i++) {
    if (sq.col[i]==MAGIC) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr << endl << "diag \\:\t";
  if (sq.diag1==MAGIC) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }

  cerr << endl << "diag /:\t";
  if (sq.diag2==MAGIC) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }

  cerr << endl<< "rows\t";

  for (int i=0; i<ORDER; i++) {
    if (sn.row[i]==NMAGIC) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr  << endl << "columns\t";
  for (int i=0; i<ORDER; i++) {
    if (sn.col[i]==NMAGIC) cerr << "@";
    else {
      cerr << "0";
      res=false;
    }
  }

  cerr << endl << "diag \\:\t";
  if (sn.diag1==NMAGIC) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }

  cerr << endl << "diag /:\t";
  if (sn.diag2==NMAGIC) cerr << "@";
  else {
    cerr << "0";
    res=false;
  }
  cerr << endl;
  return res;
}

/**
 * Rotate the square so rows become columns.
 * Note: this rotate and than invert all value at sx to dx, so we don't need
 * another temp square.
 *
 * @param sq the square
 */
void rotateSquare(square &sq) {
  word tmp;

  for (int i=0; i<ORDER-1; i++) {
    for (int j=i+1; j<ORDER; j++) {
      tmp=sq.cell[i][j];
      sq.cell[i][j]=sq.cell[j][i];
      sq.cell[j][i]=tmp;
    }
  }
}

/**
 * Swap two cell in the square
 *
 * @param sq the square where do the swap
 * @param sn the normal square
 * @param row1 the first row
 * @param col1 the first column
 * @param row2 the second row
 * @param col2 the second column
 */
inline void swap(square &sq, square &sn, int row1, int col1, int row2, int col2) {
  word tmp=sq.cell[row1][col1];
  sq.cell[row1][col1]=sq.cell[row2][col2];
  sq.cell[row2][col2]=tmp;

  tmp=sn.cell[row1][col1];
  sn.cell[row1][col1]=sn.cell[row2][col2];
  sn.cell[row2][col2]=tmp;
}

/**
 * Swap elements or rows j and i for making row i to the magic state.
 *
 * @param sq the square where working
 * @param sn the normal square
 * @param i the row to make magic
 * @param j the row where choose values
 * @return true if row becomes magic
 */
bool swapping1(square &sq, square &sn, int i, int j) {

  // this rapresent the combination of elements to choose from rows i and j
  // when we increment counter, the bits changes, so all the combination are
  // given.

  unionBits bits;

  int index[2]={i, j};                             // index for row i and j

  // try all the swap combination for making row i magic
  for (bits.counter=0; bits.counter<COMB; bits.counter++) {
    if ((SUM_CELL==MAGIC) &&
        (SUM_NCELL==NMAGIC)) {                     // are magic?
      IF_SWAP;                                     // ok, make the swaps
                                                   // that are needed
      return true;                                 // row i is now magic
    }
  }
  return false;                                    // row i is not magic
}

/**
 * Swap elements or rows j and i for making row i to the magic state.
 * Note: we don't change diagonals state.
 *
 * @param sq the square where working
 * @param sn the normal square
 * @param i the row to make magic
 * @param j the row where choose values
 * @return true is row becomes magic
 */
bool swapping2(square &sq, square &sn, int i, int j) {

  // this rapresent the combination of elements to choose from rows i and j
  // when we increment counter, the bits changes, so all the combination are
  // given.

  unionBits bits;

  int index[2]={i, j};                             // index for row i and j

  // try all the swap combination for making row i magic
  for (bits.counter=0; bits.counter<COMB; bits.counter++) {
    if ((SUM_CELL==MAGIC) &&
       (SUM_NCELL==NMAGIC)) {                     // are magic?

      if (BIT_IJ)
       continue;  // don't change elements in the diagonals

      IF_SWAP;                                     // ok, make the swaps
                                                   // that are needed
      return true;                                 // row i is now magic
    }
  }
  return false;                                    // row i is not magic
}

/**
 * Make intelligent swaps for making all rows magic
 * One rows takes value from the next until the last row.
 *
 * @param sq the square to make rows magic
 * @param sn the normal square
 * @return true if all rows become magic
 */
bool intellSwap1(square &sq, square &sn) {
  bool ok;

  for (int i=0; i<ORDER-1; i++) {
    ok=false;
    for (int j=i+1; j<ORDER; j++) {
      if (swapping1(sq, sn, i, j)) {               // swap i with all the other
        ok=true;                                   // row ok
        break;                                     // go to other rows
      }
    }
    if (!ok) return false;
  }
  return true;
}

/**
 * Make intelligent swaps for making all row magic
 *
 * @param sq the square to make rows magic
 * @param sn the normal square
 * @return true if all rows become magic
 */
bool intellSwap2(square &sq, square &sn) {
  bool ok;
  for (int i=0; i<ORDER-1; i++) {
    ok=false;
    for (int j=i+1; j<ORDER; j++) {               // swap i with all the other
      if (swapping2(sq, sn, i, j)) {
        ok=true;                                  // row ok
        break;                                    // go to other rows
      }
    }
    if (!ok) return false;                        // choose another square
  }
  return true;
}

/**
 * Make intelligent swaps for making all rows magic
 * One rows takes value from the next until the last row, but with different
 * sign from the magic state.
 *
 * @param sq the square to make rows magic
 * @param sn the normal square
 * @return true if all rows become magic
 */
bool intellSwap3(square &sq, square &sn) {
  bool ok;
  bool sign;

  sumSquare(sq);
  for (int i=0; i<ORDER-1; i++) {
    ok=false;
    sign=(sq.row[i]>MAGIC);                        // get actual sign
    for (int j=i+1; j<ORDER; j++) {
      if (((sq.row[j]>MAGIC))!=sign) {             // are different?
        if (swapping1(sq, sn, i, j)) {             // swap i with all the other
          sq.row[j]=0;
          for (int k=0; k<ORDER; k++)              // update sum
            sq.row[j]+=sq.cell[j][k];
          ok=true;                                 // row ok
          break;                                   // go to other rows
        }
      }
    }
    if (!ok) return false;
  }
  return true;
}

/**
 * Make intelligent swaps for making all rows magic
 * One rows takes value from the next until the last row, but with different
 * sign from the magic state and don't changing diagonals value.
 *
 * @param sq the square to make rows magic
 * @param sn the normal square
 * @return true if all rows become magic
 */
bool intellSwap4(square &sq, square &sn) {
  bool ok;
  bool sign;

  sumSquare(sq);
  for (int i=0; i<ORDER-1; i++) {
    ok=false;
    sign=(sq.row[i]>MAGIC);                        // get actual sign
    for (int j=i+1; j<ORDER; j++) {
      if (((sq.row[j]>MAGIC))!=sign) {             // are different?
        if (swapping2(sq, sn, i, j)) {             // swap i with all the other
          sq.row[j]=0;
          for (int k=0; k<ORDER; k++)              // update sum
            sq.row[j]+=sq.cell[j][k];
          ok=true;                                 // row ok
          break;                                   // go to other rows
        }
      }
    }
    if (!ok) return false;
  }
  return true;
}

/**
 * Permutate value for making diagonal \ magic without changing rows and
 * columns magic state.
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @param sn the normal square
 * @return true if ok
 */
bool permDiag1(int *pos, int w, square &sq, square &sn) {
  int tmp;
  int n;

  if (w>=ORDER-1) {
      if ((VALUES(sq)==MAGIC) &&
          (VALUES(sn)==NMAGIC)) {                  // row \ is magic

        square tmp;                                // tmp square
        square tmp2;

        for (int i=0; i<ORDER; i++) {
          for (int j=0; j<ORDER; j++) {            // changes the row position
            tmp.cell[i][j]=sq.cell[pos[i]][j];
            tmp2.cell[i][j]=sn.cell[pos[i]][j];
          }
        }
        sq=tmp;                                   // return the new square
        sn=tmp2;
        return true;
      }
  } else {
      for (n=w; n<ORDER; n++) {                    // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag1(pos, (w+1), sq, sn))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Permutate value for making diagonal \ magic until row / become magic too.
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @param sn the normal square
 * @return true if ok
 */
bool permDiag2(int *pos, int w, square &sq, square &sn) {
  int tmp;
  int n;

  if (w>=ORDER-1) {
      if ((VALUES(sq)==MAGIC) &&
          (VALUES(sn)==NMAGIC)) {                  // row \ is magic

        square tmp;                                // tmp square
        square tmp2;
        word sum=0;                                // sum of diagonal values
        word sum2=0;

        for (int i=0; i<ORDER; i++) {
          for (int j=0; j<ORDER; j++) {            // changes the row position
            tmp.cell[i][j]=sq.cell[pos[i]][j];
            tmp2.cell[i][j]=sn.cell[pos[i]][j];
          }
          sum+=tmp.cell[i][ORDER-1-i];
          sum2+=tmp2.cell[i][ORDER-1-i];
        }                        cerr << ((int)sum-(int)MAGIC)<< "   "<< ((int)sum2-(int)NMAGIC)<< endl;
        if ((sum==MAGIC) && (sum2==NMAGIC)) {      // are square all magic?
          sq=tmp;                                  // return the new square
          sn=tmp2;
          return true;
        }
      }
  } else {
      for (n=w; n<ORDER; n++) {                    // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag2(pos, (w+1), sq, sn))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Permutate value for making diagonal \ magic until row / become magic too, or
 * there's time out.
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @param sn the normal square
 * @return true if ok or time out
 */
bool permDiag3(int *pos, int w, square &sq, square &sn) {
  int tmp;
  int n;

  if (w>=ORDER-1) {
      timeOut++;

      if (timeOut>timeOutLimit)
        return true;                               // return true but not -1
      if ((VALUES(sq)==MAGIC) &&
          (VALUES(sn)==NMAGIC)) {                  // row \ is magic

        square tmp;                                // tmp square
        square tmp2;
        word sum=0;                                // sum of diagonal values
        word sum2=0;

        for (int i=0; i<ORDER; i++) {
          for (int j=0; j<ORDER; j++) {            // changes the row position
            tmp.cell[i][j]=sq.cell[pos[i]][j];
            tmp2.cell[i][j]=sn.cell[pos[i]][j];
          }
          sum+=tmp.cell[i][ORDER-1-i];
          sum2+=tmp2.cell[i][ORDER-1-i];
        }
        if ((sum==MAGIC) && (sum2==NMAGIC)) {      // are square all magic?
          cerr << "Time out: " << timeOut << endl;
          sq=tmp;                                  // return the new square
          sn=tmp2;
          timeOut=-1;
          return true;                             // return true with -1
        }
      }
  } else {
      for (n=w; n<ORDER; n++) {                    // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag3(pos, (w+1), sq, sn))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Permutate value for making diagonal / magic without changing \ diagonal
 *
 * @param pos contains actual position in a column of choosed values
 * @param w an index for recursion
 * @param sq the square
 * @param sn the normal square
 * @return true if ok
 */
bool permDiag4(int *pos, int w, square &sq, square &sn) {
  int tmp;
  int n;

  if (w>=ORDER-1) {
    if ((VALUES(sq)==MAGIC) &&
        (VALUES(sn)==NMAGIC)) {                    // row \ is magic

        // don't change \ diagonal
        if (POSES) {
          for (int i=0; i<ORDER; i++) {
            swap(sq, sn, pos[i], i, ORDER-1-i, i); // change only cells
          }
          return true;
        }
      }
  } else {
      for (n=w; n<ORDER; n++) {                    // calc. all the permutation
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
        if (permDiag4(pos, (w+1), sq, sn))
          return true;
        tmp=pos[n];
        pos[n]=pos[w];
        pos[w]=tmp;
      }
    }
  return false;
}

/**
 * Makes the two diagonals magic with method 1
 * Algorithm:
 * copy the half row to the diagonal \ and the half column to diagonal /
 * This changes the magic state of rows and columns.
 *
 * @param sq the square
 * @param sn the normal square
 */
void diagonal1(square &sq, square &sn) {
  for (int i=0; i<ORDER; i++) {
    swap(sq, sn, ORDER/2, i, i, i);
    swap(sq, sn, i, ORDER/2, ORDER-1-i, i);
  }
}

/**
 * Makes the diagonal \ magic and see if diagonal / become magic, or repeat
 * forever.
 *
 * @param sq the square
 * @param sn the normal square
 */
void diagonal2(square &sq, square &sn) {
  // position in a column of element to put into the diagonal
  int position[ORDER]=UP_SEQ;

  permDiag2(position, 0, sq, sn);
}

/**
 * Makes the diagonal \ magic and see if diagonal / become magic, or repeat
 * for a max time.
 *
 * @param sq the square
 * @param sn the normal square
 * @return true if build ok, false if time out
 */
bool diagonal3(square &sq, square &sn) {
  // position in a column of element to put into the diagonal
  int position[ORDER]=UP_SEQ;

  timeOut=0;
  if (permDiag3(position, 0, sq, sn))
    if (timeOut==-1)
      return true;
  return false;
}

/**
 * Makes the diagonal \ magic without changing rows and columns state, then
 * makes the other / changing columns and rows.
 *
 * @param sq the square
 * @param sn the normal square
 */
void diagonal4(square &sq, square &sn) {
  // position in a column of element to put into the diagonal
  int position1[ORDER]=UP_SEQ;
  int position2[ORDER]=DN_SEQ;

  permDiag1(position1, 0, sq, sn);
  permDiag4(position2, 0, sq, sn);
}

/**
 * Apply method A1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two magic rows
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicA1(square &sq, square &sn) {
  if (!intellSwap1(sq, sn))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap1(sq, sn))
    return false;                                  // point 3
  diagonal1(sq, sn);                               // point 4
  if (!intellSwap2(sq, sn))                        // point 5
    return false;
  rotateSquare(sq);                                // point 6
  rotateSquare(sn);
  if (!intellSwap2(sq, sn))                        // point 7
    return false;
  return true;
}

/**
 * Apply method A2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two magic rows
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 2
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicA2(square &sq, square &sn) {
  if (!intellSwap3(sq, sn))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap3(sq, sn))
    return false;                                  // point 3
  diagonal1(sq, sn);                               // point 4
  if (!intellSwap4(sq, sn))                        // point 5
    return false;
  rotateSquare(sq);                                // point 6
  rotateSquare(sn);
  if (!intellSwap4(sq, sn))                        // point 7
    return false;
  return true;
}

/**
 * Apply method A3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two magic rows
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 3
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicA3(square &sq, square &sn) {
  if (!intellSwap3(sq, sn))
    if (!intellSwap1(sq, sn))
      return false;                                // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap3(sq, sn))
    if (!intellSwap1(sq, sn))
      return false;                                // point 3
  diagonal1(sq, sn);                               // point 4
  if (!intellSwap4(sq, sn))                        // point 5
    if (!intellSwap2(sq, sn))
      return false;
  rotateSquare(sq);                                // point 6
  rotateSquare(sn);
  if (!intellSwap4(sq, sn))                        // point 7
    if (!intellSwap2(sq, sn))
      return false;
  return true;
}

/**
 * Apply method B1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make 2 rows magic
 * 2: make diagonals magic using two magic rows
 * 3: remake all rows magic without change diagonals state
 * 4: rotate the square
 * 5: remake all rows (columns in point 3) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicB1(square &sq, square &sn) {
  if (!swapping1(sq, sn, ORDER/2, 1))              // point 1
    return false;
  rotateSquare(sq);
  rotateSquare(sn);
  if (!swapping1(sq, sn, ORDER/2, 1))
    return false;
  diagonal1(sq, sn);                               // point 2
  if (!intellSwap2(sq, sn))
    return false;                                  // point 3
  rotateSquare(sq);                                // point 4
  rotateSquare(sn);
  if (!intellSwap2(sq, sn))
    return false;                                  // point 5
  return true;
}

/**
 * Apply method B2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make 2 rows magic
 * 2: make diagonals magic using two magic rows
 * 3: remake all rows magic without change diagonals state
 * 4: rotate the square
 * 5: remake all rows (columns in point 3) magic without change diagonals state
 * version 2
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicB2(square &sq, square &sn) {
  if (!swapping1(sq, sn, ORDER/2, 1))              // point 1
    return false;
  rotateSquare(sq);
  rotateSquare(sn);
  if (!swapping1(sq, sn, ORDER/2, 1))
    return false;
  diagonal1(sq, sn);                               // point 2
  if (!intellSwap4(sq, sn)) return false;          // point 3
  rotateSquare(sq);                                // point 4
  rotateSquare(sn);
  if (!intellSwap4(sq, sn)) return false;          // point 5
  return true;
}

/**
 * Apply method B3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make 2 rows magic
 * 2: make diagonals magic using two magic rows
 * 3: remake all rows magic without change diagonals state
 * 4: rotate the square
 * 5: remake all rows (columns in point 3) magic without change diagonals state
 * version 3
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicB3(square &sq, square &sn) {
  if (!swapping1(sq, sn, ORDER/2, 1))              // point 1
    return false;
  rotateSquare(sq);
  rotateSquare(sn);
  if (!swapping1(sq, sn, ORDER/2, 1))
    return false;
  diagonal1(sq, sn);                               // point 2
  if (!intellSwap4(sq, sn))
    if (!intellSwap2(sq, sn))
      return false;                                // point 3
  rotateSquare(sq);                                // point 4
  rotateSquare(sn);
  if (!intellSwap4(sq, sn))
    if (!intellSwap2(sq, sn))
      return false;                                // point 5
  return true;
}

/**
 * Apply method C1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make one diagonals magic using permutation (and pray for the other)
 * version 1
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicC1(square &sq, square &sn) {
  if (!intellSwap1(sq, sn))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap1(sq, sn))
    return false;                                  // point 3
  diagonal2(sq, sn);                               // point 4
  return true;
}

/**
 * Apply method C2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make one diagonals magic using permutation (and pray for the other)
 * version 2
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicC2(square &sq, square &sn) {
  if (!intellSwap3(sq, sn))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap3(sq, sn))
    return false;                                  // point 3
  diagonal2(sq, sn);                               // point 4
  return true;
}

/**
 * Apply method C3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make one diagonals magic using permutation (and pray for the other)
 * version 3
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicC3(square &sq, square &sn) {
  if (!intellSwap3(sq, sn))
    if (!intellSwap1(sq, sn))
      return false;                                // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap3(sq, sn))
    if (!intellSwap1(sq, sn))
      return false;                                // point 3
  if (timeOutLimit!=0) {                           // point 4
    if (!diagonal3(sq, sn))                        // use time-out
      return false;
  } else diagonal2(sq, sn);                        // don't use timeout
  return true;
}

/**
 * Apply method D1 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two algoritms
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicD1(square &sq, square &sn) {
  if (!intellSwap1(sq, sn))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap1(sq, sn))
    return false;                                  // point 3
  diagonal4(sq, sn);                               // point 4
  if (!intellSwap2(sq, sn))                        // point 5 (6, 7 not needed)
    return false;
  return true;
}

/**
 * Apply method D2 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two algoritms
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicD2(square &sq, square &sn) {
  if (!intellSwap3(sq, sn))
    return false;                                  // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap3(sq, sn))
    return false;                                  // point 3
  diagonal4(sq, sn);                               // point 4
  if (!intellSwap4(sq, sn))                        // point 5 (6, 7 not needed)
    return false;
  return true;
}

/**
 * Apply method D3 of magic solution to the passed prime square.
 * Algorithm:
 * 1: make all rows magic
 * 2: rotate the square
 * 3: make rows magic (that was columns in point 1)
 * 4: make diagonals magic using two algoritms
 * 5: remake all rows magic without change diagonals state
 * 6: rotate the square
 * 7: remake all rows (columns in point 5) magic without change diagonals state
 * version 1
 *
 * @param sq the square with prime number
 * @param sn the normal square
 * @return true if sucessful building
 */
bool magicD3(square &sq, square &sn) {
  if (!intellSwap3(sq, sn))
    if (!intellSwap1(sq, sn))
      return false;                                // point 1
  rotateSquare(sq);                                // point 2
  rotateSquare(sn);
  if (!intellSwap3(sq, sn))
    if (!intellSwap1(sq, sn))
      return false;                                // point 3
  diagonal4(sq, sn);                               // point 4
  if (!intellSwap4(sq, sn))                        // point 5 (6, 7 not needed)
    if (!intellSwap2(sq, sn))
      return false;
  return true;
}

int main(int argc, char *argv[]) {
  // result of magic building
  bool result;

  // millisecond at start time
  long double startTime;

  // millisecond at end time
  long double endTime;

  int type=43;

  // number of square to generate
  int number=1;

  if (argc>=2) {
    for (int j=1; j<argc; j++) {
      if (argv[j][0]== '-') {
        switch (argv[j][1]) {
          case 'm':
            switch (argv[j][2]) {
              case 'a':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method A1:" << endl;
                    type=11;
                    break;
                  case '2':
                    cerr << "Method A2:" << endl;
                    type=12;
                    break;
                  case '3':
                    cerr << "Method A3:" << endl;
                    type=13;
                    break;
                }
                break;
              case 'b':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method B1:" << endl;
                    type=21;
                    break;
                  case '2':
                    cerr << "Method B2:" << endl;
                    type=22;
                    break;
                  case '3':
                    cerr << "Method B3:" << endl;
                    type=23;
                    break;
                }
                break;
              case 'c':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method C1:" << endl;
                    type=31;
                    break;
                  case '2':
                    cerr << "Method C2:" << endl;
                    type=32;
                    break;
                  case '3':
                    cerr << "Method C3:" << endl;
                    type=33;
                     break;
                }
                break;
              case 'd':
                switch (argv[j][3]) {
                  case '1':
                    cerr << "Method D1:" << endl;
                    type=41;
                    break;
                  case '2':
                    cerr << "Method D2:" << endl;
                    type=42;
                    break;
                  case '3':
                    cerr << "Method D3:" << endl;
                    type=43;
                     break;
                }
                break;
            }
            break;
          case 'n':
            number=atoi(argv[j]+2);
            if (number==0) number=1;
            break;
          case 't':
            timeOutLimit=atoi(argv[j]+2);
            if (timeOutLimit<0) timeOutLimit=0;
            cerr << "Time out of " << timeOutLimit << " if allowed" << endl;
            cerr << "0 means not use time out" << endl;
            break;
          case 'h':
            cerr
            << "Usage: dpms15 [options]" << endl
            << "Options:" << endl
            << " -m<mh> where mh is the method to use (default d3):" << endl
            << "    a1: 12R 12C  2D 12R   v1\tdon't give speed result" << endl
            << "    a2: 12R 12C  2D 12R   v2\tdon't give speed result" << endl
            << "    a3: 12R 12C  2D 12R   v3\tdon't give speed result" << endl
            << "    b1:  2R  2D 12R 12C   v1\tdon't give speed result" << endl
            << "    b2:  2R  2D 12R 12C   v2\tdon't give speed result" << endl
            << "    b3:  2R  2D 12R 12C   v3\tdon't give speed result" << endl
            << "    c1: 12R 12C 1D        v1\tdon't give speed result" << endl
            << "    c2: 12R 12C 1D        v2\tdon't give speed result" << endl
            << "    c3: 12R 12C 1D        v3\tdon't give speed result" << endl
            << "        t=xxxxxx          \t don't give speed result" << endl
            << "    d1: 12R 12C 1D 1D 12R v1\tdon't give speed result" << endl
            << "    d2: 12R 12C 1D 1D 12R v2\tdon't give speed result" << endl
            << "    d3: 12R 12C 1D 1D 12R v3\tdon't give speed result" << endl
            << "    (R=row C=column D=diagonal, v version)" << endl
            << " -n<nu> where nu is the number of squares to generate "
            <<          "(default 1)" << endl
            << " -t<nn> where nn is the number of cycle for the methods that "
            <<                         "allow time out (only c3)" << endl
            << " -h     the help" << endl;
            return 0;
          default:
            cerr << "Invalid option: use dpms15 -h for help" << endl;
            return 1;
        }
      }
    }
  }

  startTime=actualTime();
  srand48((long int)startTime);
  for (int i=1; i<=number; i++) {
    cerr << "square number " << i << endl;
    do {        verifyMagic(primeSquare,normalSquare);
      eraseSquare(normalSquare);
      eraseSquare(primeSquare);

      initRandom(normalSquare);
      transformToPrime(normalSquare, primeSquare);

      switch (type) {
        case 11:
          //cerr << "A1" << endl;
          result=magicA1(primeSquare, normalSquare);
          break;
        case 12:
          //cerr << "A2" << endl;
          result=magicA2(primeSquare, normalSquare);
          break;
        case 13:
          //cerr << "A3" << endl;
          result=magicA3(primeSquare, normalSquare);
          break;
        case 21:
          //cerr << "B1" << endl;
          result=magicB1(primeSquare, normalSquare);
          break;
        case 22:
          //cerr << "B2" << endl;
          result=magicB2(primeSquare, normalSquare);
          break;
        case 23:
          //cerr << "B3" << endl;
          result=magicB3(primeSquare, normalSquare);
          break;
        case 31:
          //cerr << "C1" << endl;
          result=magicC1(primeSquare, normalSquare);
          break;
        case 32:
          //cerr << "C2" << endl;
          result=magicC2(primeSquare, normalSquare);
          break;
        case 33:
          //cerr << "C3" << endl;
          result=magicC3(primeSquare, normalSquare);
          break;
        case 41:
          //cerr << "D1" << endl;
          result=magicD1(primeSquare, normalSquare);
          break;
        case 42:
          //cerr << "D2" << endl;
          result=magicD2(primeSquare, normalSquare);
          break;
        case 43:
          //cerr << "D3" << endl;
          result=magicD3(primeSquare, normalSquare);
          break;
      }
    } while(!result);

    writeSquare(primeSquare);
    writeSquare(normalSquare);

    if (!verifyMagic(primeSquare, normalSquare))
      cout << endl << "ERROR in verify: there's an error not view before now"
           << endl;
  }

  // calculate and show statistic information
  endTime=actualTime();

  cerr << endl;
  cerr << "Total Working Time:\t\t" << ((endTime-startTime)/1000)
       << "\tseconds" << endl;
  cerr << "So in a second I can build:\t" << (1000/(endTime-startTime)*number)
       << "\tsquares" << endl;
  cerr << "or in a minute I can build:\t" << (1000*60/(endTime-startTime)*number)
       << "\tsquares" << endl;
  cerr << "in a hour I can build:\t\t" << (1000*60*60/(endTime-startTime)*number)
       << "\tsquares" << endl;
  cerr << "in a day I can build:\t\t"
       << (1000*60*60/(endTime-startTime)*24*number)
       << "\tsquares" << endl;
  cerr << "in a year I can build:\t\t"
       << (1000*60*60/(endTime-startTime)*24*365*number)
       << "\tsquares" << endl;

  return 0;
}
